@@ -26,6 +26,7 @@ from werkzeug import exceptions as wz_exc
 
 from sushy_tools.emulator import api_utils
 from sushy_tools.emulator.controllers import certificate_service as certctl
+from sushy_tools.emulator.controllers import update_service as usctl
 from sushy_tools.emulator.controllers import virtual_media as vmctl
 from sushy_tools.emulator import memoize
 from sushy_tools.emulator.resources import chassis as chsdriver
@@ -90,7 +91,7 @@ class Application(flask.Flask):
 
     def configure(self, config_file=None, extra_config=None):
         if config_file:
-            self.config.from_pyfile(config_file)
+            self.config.from_pyfile(os.path.abspath(config_file))
         if extra_config:
             self.config.update(extra_config)
 
@@ -170,6 +171,10 @@ class Application(flask.Flask):
     @property
     @memoize.memoize()
     def vmedia(self):
+        os_cloud = self.config.get('SUSHY_EMULATOR_OS_CLOUD')
+        if os_cloud:
+            return vmddriver.OpenstackDriver(self.config, self.logger,
+                                             self.systems)
         return vmddriver.StaticDriver(self.config, self.logger)
 
     @property
@@ -191,6 +196,7 @@ class Application(flask.Flask):
 app = Application()
 app.register_blueprint(certctl.certificate_service)
 app.register_blueprint(vmctl.virtual_media)
+app.register_blueprint(usctl.update_service)
 
 
 @app.errorhandler(Exception)
@@ -412,6 +418,15 @@ def system_collection_resource():
 @api_utils.returns_json
 def system_resource(identity):
     uuid = app.systems.uuid(identity)
+    try:
+        versions = app.systems.get_versions(identity)
+    except error.NotSupportedError:
+        app.logger.debug('Fetching BIOS version information not supported '
+                         'for system "%s"', identity)
+        versions = {}
+
+    bios_version = versions.get('BiosVersion')
+
     if flask.request.method == 'GET':
 
         app.logger.debug('Serving resources for system "%s"', identity)
@@ -429,6 +444,7 @@ def system_resource(identity):
             uuid=app.systems.uuid(identity),
             power_state=app.systems.get_power_state(identity),
             total_memory_gb=try_get(app.systems.get_total_memory),
+            bios_version=bios_version,
             total_cpus=try_get(app.systems.get_total_cpus),
             boot_source_target=app.systems.get_boot_device(identity),
             boot_source_mode=try_get(app.systems.get_boot_mode),
@@ -483,12 +499,17 @@ def system_resource(identity):
                     app.logger.error('Unable to record HttpBootUri for boot '
                                      'operation. Error: %s', e)
                     return 'Failed to save HttpBootUri field value.', 400
+                # Explicitly set to CD as in this case we will boot a an iso
+                # image provided, not precisely the same, but BMC facilitated
+                # HTTPBoot is a little different and the overall functionality
+                # test is more important.
+                target = 'Cd'
 
-            if target == 'UefiHttp':
-                # Reset to Cd, in our case, since we can't force override
+            if target == 'UefiHttp' and not http_uri:
+                # Reset to Pxe, in our case, since we can't force override
                 # the network boot to a specific URL. This is sort of a hack
                 # but testing functionality overall is a bit more important.
-                target = 'Cd'
+                target = 'Pxe'
 
             if target:
                 # NOTE(lucasagomes): In libvirt we always set the boot
@@ -908,6 +929,22 @@ def message_registry():
     return app.render_template('message_registry.json')
 
 
+@app.route('/redfish/v1/TaskService',
+           methods=['GET'])
+@api_utils.ensure_instance_access
+@api_utils.returns_json
+def simple_task_service():
+    return app.render_template('task_service.json')
+
+
+@app.route('/redfish/v1/TaskService/Tasks/42',
+           methods=['GET'])
+@api_utils.ensure_instance_access
+@api_utils.returns_json
+def simple_task():
+    return app.render_template('task.json')
+
+
 def parse_args():
     parser = argparse.ArgumentParser('sushy-emulator')
     parser.add_argument('--config',
